@@ -3,17 +3,19 @@
 # Use `test/fakesmsc' to test the bearerbox and the smsbox.
 
 set -e
+#set -x
 
 times=10
 interval=0
 loglevel=0
+host=127.0.0.1
 
 gw/bearerbox -v $loglevel gw/smskannel.conf > check_fakesmsc_bb.log 2>&1 &
 bbpid=$!
 
 sleep 2
 
-test/fakesmsc -i $interval -m $times '123 234 text nop' \
+test/fakesmsc -H $host -i $interval -m $times '123 234 text nop' \
     > check_fakesmsc.log 2>&1 &
 
 sleep 1
@@ -23,7 +25,7 @@ gw/smsbox -v $loglevel gw/smskannel.conf > check_fakesmsc_sms.log 2>&1 &
 running=yes
 while [ $running = yes ]
 do
-    sleep 1
+    sleep 2
     if grep "Got message $times" check_fakesmsc.log >/dev/null
     then
     	running=no
@@ -41,3 +43,5 @@ then
 fi
 
 rm check_fakesmsc*.log
+
+exit 0
\ No newline at end of file
