@@ -3,10 +3,12 @@
 # Use `test/fakewap' to test the bearerbox and the wapbox.
 
 set -e
+#set -x
 
-times=1
+host=127.0.0.1
+times=10
 port=8040
-url="http://localhost:$port/hello.wml"
+url="http://$host:$port/hello.wml"
 loglevel=0
 
 test/test_http_server -f test/hello.wml -p $port > check_http.log 2>&1 &
@@ -24,11 +26,11 @@ wappid=$!
 
 sleep 2
 
-test/fakewap -m $times $url > check_fake.log 2>&1
+test/fakewap -g $host -m $times $url > check_fake.log 2>&1
 ret=$?
 
-test/test_http -qv 4 http://localhost:$port/quit
-kill -SIGINT $bbpid $wappid
+test/test_http -qv 4 http://$host:$port/quit
+kill -INT $bbpid 
 wait
 
 if [ "$ret" != 0 ]
@@ -40,3 +42,7 @@ then
 fi
 
 rm check_bb.log check_wap.log check_fake.log check_http.log
+
+exit 0
+
+
