@@ -3,29 +3,28 @@
 # Use `test/drive_smpp' to test SMPP driver.
 
 set -e
+#set -x
 
-times=1000
-
-test/drive_smpp -v 4 -m $times -l check_smpp_drive.log &
+times=10
 
+test/drive_smpp -v 0 -m $times 2> check_smpp_drive.log & 
 sleep 1
 
-gw/bearerbox -v 4 test/drive_smpp.conf &
+gw/bearerbox -v 0 test/drive_smpp.conf 2> check_smpp_bb.log &  
 bbpid=$!
 
 running=yes
 while [ $running = yes ]
 do
     sleep 1
-    if grep "ESME has submitted all messages to SMSC." check_smpp_drive.log \
-    	>/dev/null
+    if grep "All messages sent to ESME." check_smpp_drive.log > /dev/null  
     then
         running=no
     fi
 done
+sleep 5
 
-kill $bbpid
-wait
+kill -INT $bbpid
 
 if grep 'WARNING:|ERROR:|PANIC:' check_smpp*.log >/dev/null
 then
@@ -34,4 +33,13 @@ then
         exit 1
 fi
 
-rm check_smpp_drive.log check_smpp_bb.log
+rm -f check_smpp*.log 
+
+exit 0
+
+
+
+
+
+
+
