@@ -1,6 +1,66 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * Wap_push_ota.c: implementation of push related requests of OTA protocol
  *
+ * This module implements requirement primitives of WAP-189-PushOTA-20000217-a
+ * (hereafter called ota).
+ * In addition, WAP-203-WSP-20000504-a (wsp) is referred.
+ *
  * This module forwards push  requests made by the wap_push_ppg module to 
  * connected or connectionless session services.
  * Indications (for confirmed push, push abort and disconnect, e.g., in the 
@@ -12,7 +72,7 @@
  * into WSP format and unpacking them from WSP format. This module contains
  * decoding part.
  *
- * By Aarno Syvnen for Wapit Ltd
+ * By Aarno Syvnen for Wapit Ltd and Global Networks Inc.
  */
 
 #include <errno.h>
@@ -82,7 +142,7 @@ static List *add_push_flag(WAPEvent *e);
 static void check_session_request_headers(List *headers);
 
 /*
- * Contact points and appication ids in the push initiator are packed into a
+ * Contact points and application ids in the push initiator are packed into a
  * specific structure, being like WSP PDUs.
  */
 static Octstr *pack_sia(List *headers);
@@ -106,8 +166,8 @@ static void bearerbox_address_destroy(BearerboxAddress *ba);
 void wap_push_ota_init(wap_dispatch_func_t *wsp_dispatch,
                        wap_dispatch_func_t *wsp_unit_dispatch)
 {
-    ota_queue = list_create();
-    list_add_producer(ota_queue);
+    ota_queue = gwlist_create();
+    gwlist_add_producer(ota_queue);
 
     dispatch_to_wsp = wsp_dispatch;
     dispatch_to_wsp_unit = wsp_unit_dispatch;
@@ -123,17 +183,17 @@ void wap_push_ota_shutdown(void)
 {
     gw_assert(run_status == running);
     run_status = terminating;
-    list_remove_producer(ota_queue);
+    gwlist_remove_producer(ota_queue);
     gwthread_join_every(main_thread);
 
-    list_destroy(ota_queue, wap_event_destroy_item);
+    gwlist_destroy(ota_queue, wap_event_destroy_item);
     bearerbox_address_destroy(bearerbox);
 }
 
 void wap_push_ota_dispatch_event(WAPEvent *e)
 {
     gw_assert(run_status == running); 
-    list_produce(ota_queue, e);
+    gwlist_produce(ota_queue, e);
 }
 
 /*
@@ -158,7 +218,7 @@ static void main_thread(void *arg)
 {
     WAPEvent *e;
 
-    while (run_status == running && (e = list_consume(ota_queue)) != NULL) {
+    while (run_status == running && (e = gwlist_consume(ota_queue)) != NULL) {
         handle_ota_event(e);
     } 
 
@@ -209,6 +269,27 @@ static void make_session_request(WAPEvent *e)
     check_session_request_headers(push_headers);
 
     wsp_event = wap_event_create(S_Unit_Push_Req);
+    wsp_event->u.S_Unit_Push_Req.address_type =
+        e->u.Pom_SessionRequest_Req.address_type;
+    if (e->u.Pom_SessionRequest_Req.smsc_id != NULL)
+        wsp_event->u.S_Unit_Push_Req.smsc_id =
+            octstr_duplicate(e->u.Pom_SessionRequest_Req.smsc_id);
+    else
+        wsp_event->u.S_Unit_Push_Req.smsc_id = NULL;
+    if (e->u.Pom_SessionRequest_Req.dlr_url != NULL)
+        wsp_event->u.S_Unit_Push_Req.dlr_url =
+            octstr_duplicate(e->u.Pom_SessionRequest_Req.dlr_url);
+    else
+        wsp_event->u.S_Unit_Push_Req.dlr_url = NULL;
+    wsp_event->u.S_Unit_Push_Req.dlr_mask = e->u.Pom_SessionRequest_Req.dlr_mask;
+    if (e->u.Pom_SessionRequest_Req.smsbox_id != NULL)
+        wsp_event->u.S_Unit_Push_Req.smsbox_id =
+            octstr_duplicate(e->u.Pom_SessionRequest_Req.smsbox_id);
+    else
+        wsp_event->u.S_Unit_Push_Req.smsbox_id = NULL;    
+    wsp_event->u.S_Unit_Push_Req.service_name = 
+        octstr_duplicate(e->u.Pom_SessionRequest_Req.service_name);
+
     wsp_event->u.S_Unit_Push_Req.push_id = 
         e->u.Pom_SessionRequest_Req.push_id;
     wsp_event->u.S_Unit_Push_Req.addr_tuple = 
@@ -260,7 +341,7 @@ static void make_confirmed_push_request(WAPEvent *e)
 
     if (e->u.Po_ConfirmedPush_Req.push_body != NULL)
         wsp_event->u.S_ConfirmedPush_Req.push_body =
-	  octstr_duplicate(e->u.Po_ConfirmedPush_Req.push_body);
+	        octstr_duplicate(e->u.Po_ConfirmedPush_Req.push_body);
     else
         wsp_event->u.S_ConfirmedPush_Req.push_body = NULL;
      
@@ -275,8 +356,21 @@ static void make_unit_push_request(WAPEvent *e)
 {
     WAPEvent *wsp_event;
     List *push_headers;
+    Octstr *smsc_id;
+    Octstr *dlr_url;
+    Octstr *smsbox_id;
+    Octstr *push_body;
+    Octstr *service_name;
 
     gw_assert(e->type == Po_Unit_Push_Req);
+    gw_assert(e->u.Po_Unit_Push_Req.addr_tuple);
+    gw_assert(e->u.Po_Unit_Push_Req.service_name);
+
+    smsc_id = octstr_duplicate(e->u.Po_Unit_Push_Req.smsc_id); 
+    dlr_url = octstr_duplicate(e->u.Po_Unit_Push_Req.dlr_url);
+    smsbox_id = octstr_duplicate(e->u.Po_Unit_Push_Req.smsbox_id);
+    push_body = octstr_duplicate(e->u.Po_Unit_Push_Req.push_body);
+    service_name = octstr_duplicate(e->u.Po_Unit_Push_Req.service_name); 
     push_headers = add_push_flag(e);
 
     wsp_event = wap_event_create(S_Unit_Push_Req);
@@ -285,9 +379,24 @@ static void make_unit_push_request(WAPEvent *e)
     wsp_event->u.S_Unit_Push_Req.push_id = e->u.Po_Unit_Push_Req.push_id;
     wsp_event->u.S_Unit_Push_Req.push_headers = push_headers;
 
-    if (e->u.Po_Unit_Push_Req.push_body != NULL)
-        wsp_event->u.S_Unit_Push_Req.push_body =
-	    octstr_duplicate(e->u.Po_Unit_Push_Req.push_body);
+    wsp_event->u.S_Unit_Push_Req.address_type = 
+        e->u.Po_Unit_Push_Req.address_type;
+    if (smsc_id != NULL)
+        wsp_event->u.S_Unit_Push_Req.smsc_id = smsc_id;
+    else
+        wsp_event->u.S_Unit_Push_Req.smsc_id = NULL;  
+    if (dlr_url != NULL)      
+        wsp_event->u.S_Unit_Push_Req.dlr_url = dlr_url;        
+    else  
+        wsp_event->u.S_Unit_Push_Req.dlr_url = NULL;      
+    wsp_event->u.S_Unit_Push_Req.dlr_mask = e->u.Po_Unit_Push_Req.dlr_mask;            
+    if (smsbox_id != NULL)
+        wsp_event->u.S_Unit_Push_Req.smsbox_id = smsbox_id;
+    else
+        wsp_event->u.S_Unit_Push_Req.smsbox_id = NULL;
+    wsp_event->u.S_Unit_Push_Req.service_name = service_name;
+    if (push_body != NULL)
+        wsp_event->u.S_Unit_Push_Req.push_body = push_body;
     else
         wsp_event->u.S_Unit_Push_Req.push_body = NULL;
 
@@ -315,7 +424,8 @@ static void abort_push(WAPEvent *e)
 }
 
 /*
- * Add push flag into push headers. Push flag is defined in PushOTA, p. 17-18.
+ * Add push flag into push headers. Push flag is defined in ota, p. 17-18.
+ * If there is no flags set, no Push-Flag header is added.
  */
 static List *add_push_flag(WAPEvent *e)
 {
@@ -360,9 +470,11 @@ static List *add_push_flag(WAPEvent *e)
     push_flag = 0;
     push_flag = push_flag | authenticated | trusted | last;
     
+    if (push_flag) {
     buf = octstr_format("%d", push_flag);
     http_header_add(headers, "Push-Flag", octstr_get_cstr(buf)); 
     octstr_destroy(buf);
+    }
 
     return headers;
 }
@@ -396,7 +508,7 @@ static void flags_assert(WAPEvent *e)
 }
 
 /*
- * Accepted reasons are defined in OTA 6.3.3.
+ * Accepted reasons are defined in ota 6.3.3.
  */
 static void reason_assert(long reason)
 {
@@ -407,7 +519,7 @@ static void reason_assert(long reason)
 
 /*
  * When server is requesting a session with a client, content type and applic-
- * ation headers must be present (this behaviour is defined in PushOTA, p. 14).
+ * ation headers must be present (this behaviour is defined in ota, p. 14).
  * We check headers for them and add them if they are not already present. 
  * X-WAP-Application-Id has been added by ppg module.
  */
@@ -419,7 +531,7 @@ static void check_session_request_headers(List *headers)
 
 /*
  * Pack contact points and application id list into sia content type. It is 
- * defined in PushOTA, p. 18. 
+ * defined in ota, p. 18. 
  */
 static Octstr *pack_sia(List *headers)
 {
@@ -440,13 +552,12 @@ static Octstr *pack_sia(List *headers)
 }
 
 /*
- * Turns list of X-Wap-Application-Id headers into the numeric form.
+ * Input: List of headers containing only X-Wap-Application-Id headers, values
+ * being numeric application id codes. (Ppg module does coding of the header 
+ * value part of the X-WAP-Application-Id header).
+ * Output: Octstr containing them in a byte list (one id per byte). 
  *
- * Input: List of headers containing only X-Wap-Application-Id headers
- * Output: Octstr containing them in a numeric format. (Ppg module does coding
- * of the header value part of the X-WAP-Application-Id header).
- *
- * Returns: Octstr containing headers, if succesful, otherwise an empty 
+ * Returns: Octstr containing headers, if succesfull, otherwise an empty 
  * octstr.
  */
 static Octstr *pack_appid_list(List *headers)
@@ -454,15 +565,16 @@ static Octstr *pack_appid_list(List *headers)
     Octstr *appid_os,
            *header_name,
            *header_value;
-    long i,
-         j;
+    long i;
+    size_t len;
 
-    i = j = 0;
+    i = 0;
     appid_os = octstr_create("");
+    len = (size_t) gwlist_len(headers);
 
-    gw_assert(list_len(headers));
+    gw_assert(len);
 
-    while (i < list_len(headers)) {
+    while (i < len) {
         http_header_get(headers, i, &header_name, &header_value);
         gw_assert(octstr_compare(header_name, 
                   octstr_imm("X-WAP-Application-Id")) == 0);
@@ -477,7 +589,7 @@ static Octstr *pack_appid_list(List *headers)
 
 /*
  * NB: This data includes bearer information. We use IPv4 values. Address Type
- * is defined in WSP, table 16, p. 65
+ * is defined in wsp, table 16, p. 65
  */
 static Octstr *pack_server_address(void)
 {
@@ -514,9 +626,9 @@ static Octstr *pack_server_address(void)
 static Octstr *name(Octstr *in)
 {
     if (octstr_compare(in, octstr_imm("localhost")) != 0)
-	return octstr_duplicate(in);
+    	return octstr_duplicate(in);
     else
-	return octstr_duplicate(get_official_ip());
+	    return octstr_duplicate(get_official_ip());
 }
 
 static BearerboxAddress *bearerbox_address_create(void) 
@@ -535,7 +647,6 @@ static void bearerbox_address_destroy(BearerboxAddress *ba)
     if (ba == NULL)
         return;
 
-    mutex_lock(ba->mutex);
     octstr_destroy(ba->address);
     mutex_destroy(ba->mutex);
     gw_free(ba);
