@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * gwmem-check.c - memory management wrapper functions, check flavor
  *
@@ -29,12 +85,17 @@
  * after it's been freed.
  * 
  * Richard Braakman
+ * Alexander Malysh (added backtrace support)
  */
 
+#include "gw-config.h"
 
 #include <stdlib.h>
 #include <errno.h>
 #include <string.h>
+#if HAVE_BACKTRACE
+#include <execinfo.h>
+#endif
 
 #include "gwlib.h"
 
@@ -90,6 +151,10 @@ struct area
     struct location allocator;     /* Caller that alloced area */
     struct location reallocator;   /* Caller that last realloced area */
     struct location claimer;       /* Owner of area, set by caller */
+#if HAVE_BACKTRACE
+    void *frames[10]; /* 10 callers should be sufficient */
+    size_t frame_size;
+#endif
 };
 
 /* Number of bytes to reserve on either side of each allocated area,
@@ -97,7 +162,8 @@ struct area
  * enough to hold a long. */
 #define MARKER_SIZE 16
 
-#define MAX_TAB_SIZE (1024*1024L)
+/* 100 MB */
+#define MAX_TAB_SIZE (100*1024*1024L)
 #define MAX_ALLOCATIONS ((long) (MAX_TAB_SIZE/sizeof(struct area)))
 
 /* Freed areas are thrown into the free ring.  They are not released
@@ -128,12 +194,12 @@ static long total_size;
 
 /* Static functions */
 
-static void lock(void)
+static inline void lock(void)
 {
     mutex_lock(&gwmem_lock);
 }
 
-static void unlock(void)
+static inline void unlock(void)
 {
     mutex_unlock(&gwmem_lock);
 }
@@ -185,7 +251,7 @@ static int untouched(unsigned char *p, size_t bytes, long pattern)
 }
 
 /* Fill the end marker for this area */
-static void endmark(unsigned char *p, size_t size)
+static inline void endmark(unsigned char *p, size_t size)
 {
     fill(p + size, MARKER_SIZE, END_MARK_PATTERN);
 }
@@ -243,35 +309,49 @@ static void dump_area(struct area *area)
 {
     debug("gwlib.gwmem", 0, "Area %p, size %ld, max_size %ld",
           area->area, (long) area->area_size, (long) area->max_size);
-    debug("gwlib.gwmem", 0, "Allocated by %s at %s:%ld",
+    debug("gwlib.gwmem", 0, "Allocated by %s() at %s:%ld",
           area->allocator.function,
           area->allocator.filename,
           area->allocator.lineno);
     if (area->reallocator.function) {
-        debug("gwlib.gwmem", 0, "Re-allocated by %s at %s:%ld",
+        debug("gwlib.gwmem", 0, "Re-allocated by %s() at %s:%ld",
               area->reallocator.function,
               area->reallocator.filename,
               area->reallocator.lineno);
     }
     if (area->claimer.function) {
-        debug("gwlib.gwmem", 0, "Claimed by %s at %s:%ld",
+        debug("gwlib.gwmem", 0, "Claimed by %s() at %s:%ld",
               area->claimer.function,
               area->claimer.filename,
               area->claimer.lineno);
     }
     if (area->area_size > 0) {
-	size_t i;
-	unsigned char *p;
-	char buf[MAX_DUMP * 3 + 1];
-	
-    	p = area->area;
-	buf[0] = '\0';
-	for (i = 0; i < area->area_size && i < MAX_DUMP; ++i)
-	    sprintf(strchr(buf, '\0'), "%02x ", p[i]);
-    	debug("gwlib.gwmem", 0, "Contents of area (first %d bytes):",
-	      MAX_DUMP);
-	debug("gwlib.gwmem", 0, "  %s", buf);
+        size_t i;
+        unsigned char *p;
+        char buf[MAX_DUMP * 3 + 1];
+
+        p = area->area;
+        buf[0] = '\0';
+        for (i = 0; i < area->area_size && i < MAX_DUMP; ++i)
+            sprintf(strchr(buf, '\0'), "%02x ", p[i]);
+
+        debug("gwlib.gwmem", 0, "Contents of area (first %d bytes):", MAX_DUMP);
+        debug("gwlib.gwmem", 0, "  %s", buf);
     }
+#if HAVE_BACKTRACE
+    {
+        size_t i;
+        char **strings = backtrace_symbols(area->frames, area->frame_size);
+        debug("gwlib.gwmem", 0, "Backtrace of last malloc/realloc:");
+        for (i = 0; i < area->frame_size; i++) {
+            if (strings != NULL)
+                debug("gwlib.gwmem", 0, "%s", strings[i]);
+            else
+                debug("gwlib.gwmem", 0, "%p", area->frames[i]);
+        }
+        free(strings);
+    }
+#endif
 }
 
 static struct area *find_area(unsigned char *p)
@@ -352,6 +432,9 @@ static struct area *record_allocation(unsigned char *p, size_t size,
     area->allocator.filename = filename;
     area->allocator.lineno = lineno;
     area->allocator.function = function;
+#if HAVE_BACKTRACE
+    area->frame_size = backtrace(area->frames, sizeof(area->frames) / sizeof(void*));
+#endif
 
     startmark(area->area, num_allocations);
     endmark(area->area, area->area_size);
@@ -439,7 +522,7 @@ void *gw_check_malloc(size_t size, const char *filename, long lineno,
 
     p = malloc(size + 2 * MARKER_SIZE);
     if (p == NULL)
-        panic(errno, "Memory allocation of %d bytes failed.", size);
+        panic(errno, "Memory allocation of %ld bytes failed.", (long)size);
     p += MARKER_SIZE;
 
     lock();
@@ -585,6 +668,7 @@ void gw_check_check_leaks(void)
     }
     gw_assert(calculated_size == total_size);
 
+    debug("gwlib.gwmem", 0, "----------------------------------------");
     debug("gwlib.gwmem", 0, "Current allocations: %ld areas, %ld bytes",
           num_allocations, total_size);
     debug("gwlib.gwmem", 0, "Highest number of allocations: %ld areas",
