@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * gwthread-pthread.c - implementation of gwthread.h using POSIX threads.
  *
@@ -8,9 +64,14 @@
 #include <errno.h>
 #include <pthread.h>
 #include <signal.h>
+#include <string.h>
 
 #include "gwlib/gwlib.h"
 
+#ifdef HAVE_LIBSSL
+#include <openssl/err.h>
+#endif /* HAVE_LIBSSL */
+
 /* Maximum number of live threads we can support at once.  Increasing
  * this will increase the size of the threadtable.  Use powers of two
  * for efficiency. */
@@ -28,6 +89,7 @@ struct threadinfo
      * to register.  This is safe because the thread table is always
      * locked when a thread accesses this field. */
     List *joiners;
+    pid_t pid;
 };
 
 struct new_thread_args
@@ -35,6 +97,8 @@ struct new_thread_args
     gwthread_func_t *func;
     void *arg;
     struct threadinfo *ti;
+    /* signals already started thread to die */
+    int failed;
 };
 
 /* The index is the external thread number modulo the table size; the
@@ -60,11 +124,11 @@ static struct threadinfo mainthread;
 /* Our key for accessing the (struct gwthread *) we stash in the
  * thread-specific-data area.  This is much more efficient than
  * accessing a global table, which we would have to lock. */
-pthread_key_t tsd_key;
+static pthread_key_t tsd_key;
 
-pthread_mutex_t threadtable_lock;
+static pthread_mutex_t threadtable_lock;
 
-static void lock(void)
+static void inline lock(void)
 {
     int ret;
 
@@ -74,7 +138,7 @@ static void lock(void)
     }
 }
 
-static void unlock(void)
+static void inline unlock(void)
 {
     int ret;
 
@@ -109,18 +173,24 @@ static long fill_threadinfo(pthread_t id, const char *name,
 
     gw_assert(active_threads < THREADTABLE_SIZE);
 
+    /* initialize to default values */
     ti->self = id;
     ti->name = name;
     ti->func = func;
+    ti->pid = -1;
+    ti->wakefd_recv = -1;
+    ti->wakefd_send = -1;
+    ti->joiners = NULL;
+    ti->number = -1;
 
     if (pipe(pipefds) < 0) {
-        panic(errno, "cannot allocate wakeup pipe for new thread");
+        error(errno, "cannot allocate wakeup pipe for new thread");
+        return -1;
     }
     ti->wakefd_recv = pipefds[0];
     ti->wakefd_send = pipefds[1];
     socket_set_blocking(ti->wakefd_recv, 0);
     socket_set_blocking(ti->wakefd_send, 0);
-    ti->joiners = NULL;
 
     /* Find a free table entry and claim it. */
     first_try = next_threadnumber;
@@ -128,8 +198,9 @@ static long fill_threadinfo(pthread_t id, const char *name,
         ti->number = next_threadnumber++;
         /* Check if we looped all the way around the thread table. */
         if (ti->number == first_try + THREADTABLE_SIZE) {
-            panic(0, "Cannot have more than %d active threads",
-                  THREADTABLE_SIZE);
+            error(0, "Cannot have more than %d active threads", THREADTABLE_SIZE);
+            ti->number = -1;
+            return -1;
         }
     } while (THREAD(ti->number) != NULL);
     THREAD(ti->number) = ti;
@@ -166,7 +237,7 @@ static void alert_joiners(void)
     threadinfo = getthreadinfo();
     if (!threadinfo->joiners)
         return;
-    while ((joiner_cond = list_extract_first(threadinfo->joiners))) {
+    while ((joiner_cond = gwlist_extract_first(threadinfo->joiners))) {
         pthread_cond_broadcast(joiner_cond);
     }
 }
@@ -176,26 +247,19 @@ static void delete_threadinfo(void)
     struct threadinfo *threadinfo;
 
     threadinfo = getthreadinfo();
-    list_destroy(threadinfo->joiners, NULL);
-    close(threadinfo->wakefd_recv);
-    close(threadinfo->wakefd_send);
-    THREAD(threadinfo->number) = NULL;
-    active_threads--;
+    gwlist_destroy(threadinfo->joiners, NULL);
+    if (threadinfo->wakefd_recv != -1)
+        close(threadinfo->wakefd_recv);
+    if (threadinfo->wakefd_send != -1)
+        close(threadinfo->wakefd_send);
+    if (threadinfo->number != -1) {
+        THREAD(threadinfo->number) = NULL;
+        active_threads--;
+    }
     gw_assert(threadinfo != &mainthread);
     gw_free(threadinfo);
 }
 
-static void create_threadinfo_main(void)
-{
-    int ret;
-
-    fill_threadinfo(pthread_self(), "main", NULL, &mainthread);
-    ret = pthread_setspecific(tsd_key, &mainthread);
-    if (ret != 0) {
-        panic(ret, "gwthread-pthread: pthread_setspecific failed");
-    }
-}
-
 void gwthread_init(void)
 {
     int ret;
@@ -213,7 +277,13 @@ void gwthread_init(void)
     }
     active_threads = 0;
 
-    create_threadinfo_main();
+    /* create main thread info */
+    if (fill_threadinfo(pthread_self(), "main", NULL, &mainthread) == -1)
+        panic(0, "gwthread-pthread: unable to fill main threadinfo.");
+
+    ret = pthread_setspecific(tsd_key, &mainthread);
+    if (ret != 0)
+        panic(ret, "gwthread-pthread: pthread_setspecific failed");
 }
 
 /* Note that the gwthread library can't shut down completely, because
@@ -263,6 +333,15 @@ static void *new_thread(void *arg)
     /* Make sure we don't start until our parent has entered
      * our thread info in the thread table. */
     lock();
+    /* check for initialization errors */
+    if (p->failed) {
+        /* Must free p before signaling our exit, otherwise there is
+        * a race with gw_check_leaks at shutdown. */
+        gw_free(p);
+        delete_threadinfo();
+        unlock();
+        return NULL;
+    }
     unlock();
 
     /* This has to be done here, because pthread_setspecific cannot
@@ -276,12 +355,21 @@ static void *new_thread(void *arg)
         panic(ret, "gwthread-pthread: pthread_setspecific failed");
     }
 
+    p->ti->pid = getpid();
+    debug("gwlib.gwthread", 0, "Thread %ld (%s) maps to pid %ld.",
+          p->ti->number, p->ti->name, (long) p->ti->pid);
+
     (p->func)(p->arg);
 
     lock();
     debug("gwlib.gwthread", 0, "Thread %ld (%s) terminates.",
           p->ti->number, p->ti->name);
     alert_joiners();
+#ifdef HAVE_LIBSSL
+    /* Clear the OpenSSL thread-specific error queue to avoid
+     * memory leaks. */
+    ERR_remove_state(gwthread_self());
+#endif /* HAVE_LIBSSL */
     /* Must free p before signaling our exit, otherwise there is
      * a race with gw_check_leaks at shutdown. */
     gw_free(p);
@@ -297,6 +385,15 @@ static void *new_thread(void *arg)
  * *old_set_storage.
  * Return 0 for success, or -1 if an error occurred.
  */
+ 
+ /* 
+  * This does not work in Darwin alias MacOS X alias Mach kernel,
+  * however. So we define a dummy function doing nothing.
+  */
+#if defined(DARWIN_OLD)
+    static int pthread_sigmask();
+#endif
+  
 static int block_user_signals(sigset_t *old_set_storage)
 {
     int ret;
@@ -305,7 +402,7 @@ static int block_user_signals(sigset_t *old_set_storage)
     ret = sigemptyset(&block_signals);
     if (ret != 0) {
         error(errno, "gwthread-pthread: Couldn't initialize signal set");
-	return -1;
+	    return -1;
     }
     ret = sigaddset(&block_signals, SIGHUP);
     ret |= sigaddset(&block_signals, SIGTERM);
@@ -313,7 +410,7 @@ static int block_user_signals(sigset_t *old_set_storage)
     ret |= sigaddset(&block_signals, SIGINT);
     if (ret != 0) {
         error(0, "gwthread-pthread: Couldn't add signal to signal set");
-	return -1;
+	    return -1;
     }
     ret = pthread_sigmask(SIG_BLOCK, &block_signals, old_set_storage);
     if (ret != 0) {
@@ -334,6 +431,7 @@ static void restore_user_signals(sigset_t *old_set)
     }
 }
 
+
 static long spawn_thread(gwthread_func_t *func, const char *name, void *arg)
 {
     int ret;
@@ -348,6 +446,7 @@ static long spawn_thread(gwthread_func_t *func, const char *name, void *arg)
     p->func = func;
     p->arg = arg;
     p->ti = gw_malloc(sizeof(*(p->ti)));
+    p->failed = 0;
 
     /* Lock the thread table here, so that new_thread can block
      * on that lock.  That way, the new thread won't start until
@@ -370,13 +469,18 @@ static long spawn_thread(gwthread_func_t *func, const char *name, void *arg)
     }
     ret = pthread_detach(id);
     if (ret != 0) {
-        warning(ret, "Could not detach new thread.");
+        error(ret, "Could not detach new thread.");
     }
 
     new_thread_id = fill_threadinfo(id, name, func, p->ti);
+    if (new_thread_id == -1)
+        p->failed = 1;
     unlock();
     
-    debug("gwlib.gwthread", 0, "Started thread %ld (%s)", new_thread_id, name);
+    if (new_thread_id != -1)
+        debug("gwlib.gwthread", 0, "Started thread %ld (%s)", new_thread_id, name);
+    else
+        debug("gwlib.gwthread", 0, "Failed to start thread (%s)", name);
 
     return new_thread_id;
 }
@@ -399,7 +503,7 @@ long gwthread_create_real(gwthread_func_t *func, const char *name, void *arg)
      * should be queued by the operating system.
      */
     if (gwthread_self() == MAIN_THREAD_ID)
-	sigtrick = block_user_signals(&old_signal_set) == 0;
+	    sigtrick = block_user_signals(&old_signal_set) == 0;
 
     thread_id = spawn_thread(func, name, arg);
 
@@ -409,7 +513,7 @@ long gwthread_create_real(gwthread_func_t *func, const char *name, void *arg)
      * the old one back.
      */
     if (sigtrick)
- 	restore_user_signals(&old_signal_set);
+ 	    restore_user_signals(&old_signal_set);
     
     return thread_id;
 }
@@ -441,8 +545,8 @@ void gwthread_join(long thread)
     }
 
     if (!threadinfo->joiners)
-        threadinfo->joiners = list_create();
-    list_append(threadinfo->joiners, &exit_cond);
+        threadinfo->joiners = gwlist_create();
+    gwlist_append(threadinfo->joiners, &exit_cond);
 
     /* The wait immediately releases the lock, and reacquires it
      * when the condition is satisfied.  So don't worry, we're not
@@ -507,8 +611,8 @@ void gwthread_join_every(gwthread_func_t *func)
               "Waiting for %ld (%s) to terminate",
               ti->number, ti->name);
         if (!ti->joiners)
-            ti->joiners = list_create();
-        list_append(ti->joiners, &exit_cond);
+            ti->joiners = gwlist_create();
+        gwlist_append(ti->joiners, &exit_cond);
         ret = pthread_cond_wait(&exit_cond, &threadtable_lock);
         if (ret != 0)
             warning(ret, "gwthread_join_all: error in pthread_cond_wait");
@@ -523,12 +627,36 @@ long gwthread_self(void)
 {
     struct threadinfo *threadinfo;
     threadinfo = pthread_getspecific(tsd_key);
-    if (threadinfo)
+    if (threadinfo) 
         return threadinfo->number;
     else
         return -1;
 }
 
+/* Return the thread pid of this thread. */
+long gwthread_self_pid(void)
+{
+    struct threadinfo *threadinfo;
+    threadinfo = pthread_getspecific(tsd_key);
+    if (threadinfo && threadinfo->pid != -1) 
+        return (long) threadinfo->pid;
+    else
+        return (long) getpid();
+}
+
+void gwthread_self_ids(long *tid, long *pid)
+{
+    struct threadinfo *threadinfo;
+    threadinfo = pthread_getspecific(tsd_key);
+    if (threadinfo) {
+        *tid = threadinfo->number;
+        *pid = (threadinfo->pid != -1) ? threadinfo->pid : getpid();
+    } else {
+        *tid = -1;
+        *pid = getpid();
+    }
+}
+
 void gwthread_wakeup(long thread)
 {
     unsigned char c = 0;
@@ -562,9 +690,11 @@ int gwthread_pollfd(int fd, int events, double timeout)
 
     pollfd[0].fd = threadinfo->wakefd_recv;
     pollfd[0].events = POLLIN;
+    pollfd[0].revents = 0;
 
     pollfd[1].fd = fd;
     pollfd[1].events = events;
+    pollfd[1].revents = 0;
 
     milliseconds = timeout * 1000;
     if (milliseconds < 0)
@@ -598,6 +728,7 @@ int gwthread_poll(struct pollfd *fds, long numfds, double timeout)
     pollfds = gw_malloc((numfds + 1) * sizeof(*pollfds));
     pollfds[0].fd = threadinfo->wakefd_recv;
     pollfds[0].events = POLLIN;
+    pollfds[0].revents = 0;
     memcpy(pollfds + 1, fds, numfds * sizeof(*pollfds));
 
     milliseconds = timeout * 1000;
@@ -650,6 +781,56 @@ void gwthread_sleep(double seconds)
 }
 
 
+void gwthread_sleep_micro(double dseconds)
+{
+    fd_set fd_set_recv;
+    struct threadinfo *threadinfo;
+    int fd;
+    int ret;
+
+    threadinfo = getthreadinfo();
+    fd = threadinfo->wakefd_recv;
+
+    FD_ZERO(&fd_set_recv);
+    FD_SET(fd, &fd_set_recv);
+
+    if (dseconds < 0) {
+        ret = select(fd + 1, &fd_set_recv, NULL, NULL, NULL);
+    } else {
+        struct timeval timeout;
+        timeout.tv_sec = dseconds;
+        timeout.tv_usec = (dseconds - timeout.tv_sec) * 1000000;
+
+        ret = select(fd + 1, &fd_set_recv, NULL, NULL, &timeout);
+    }
+
+    if (ret < 0) {
+        if (errno != EINTR && errno != EAGAIN) {
+            warning(errno, "gwthread_sleep_micro: error in select()");
+        }
+    }
+
+    if (FD_ISSET(fd, &fd_set_recv)) {
+        flushpipe(fd);
+    }
+}
+
+
+int gwthread_cancel(long thread)
+{
+    struct threadinfo *threadinfo;
+    
+    gw_assert(thread >= 0);
+    
+    threadinfo = THREAD(thread);
+    if (threadinfo == NULL || threadinfo->number != thread) {
+        return -1;
+    } else {
+        return pthread_cancel(threadinfo->self);
+    }
+}
+
+
 #ifndef BROKEN_PTHREADS
 
 /* Working pthreads */
@@ -658,7 +839,7 @@ int gwthread_shouldhandlesignal(int signal){
 }
 #else
 
-/* Somewhat broken pthreads */
+/* Somewhat broken pthreads */ 
 int gwthread_shouldhandlesignal(int signal){
     return (gwthread_self() == MAIN_THREAD_ID);
 }
@@ -670,18 +851,27 @@ int gwthread_dumpsigmask(void) {
 
     /* Grab the signal set data from our thread */
     if (pthread_sigmask(SIG_BLOCK, NULL, &signal_set) != 0) {
-	warning(0, "gwthread_dumpsigmask: Couldn't get signal mask.");
-	return -1;
+	    warning(0, "gwthread_dumpsigmask: Couldn't get signal mask.");
+	    return -1;
     }
     
     /* For each signal normally defined (there are usually only 32),
      * print a message if we don't block it. */
     for (signum = 1; signum <= 32; signum++) {
-	if (!sigismember(&signal_set, signum)) {
-	    debug("gwlib", 0,
-		"gwthread_dumpsigmask: Signal Number %d will be caught.", 
-		signum);
-	}
+	     if (!sigismember(&signal_set, signum)) {
+	         debug("gwlib", 0,
+		     "gwthread_dumpsigmask: Signal Number %d will be caught.", 
+		     signum);
+	     }
     }
     return 0;
 }
+
+/* DARWIN alias MacOS X doesnt have pthread_sigmask in its pthreads implementation */
+
+#if defined(DARWIN_OLD)
+static int pthread_sigmask()
+{
+    return 0;
+}
+#endif
