@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * http.h - HTTP protocol implementation
  *
@@ -76,27 +132,52 @@
 
 
 /*
- * Well-known return values from HTTP servers. This is not a complete
- * list, but it includes the values that Kannel needs to handle
- * specially.
+ * Well-known return values from HTTP servers. This is a complete
+ * list as defined by the W3C in RFC 2616, section 10.4.3.
  */
 
 enum {
-	HTTP_OK				= 200,
-	HTTP_CREATED			= 201,
-	HTTP_ACCEPTED			= 202,
-	HTTP_NO_CONTENT			= 204,
-	HTTP_RESET_CONTENT		= 205,
-	HTTP_MOVED_PERMANENTLY		= 301,
-	HTTP_FOUND			= 302,
-	HTTP_SEE_OTHER			= 303,
-	HTTP_NOT_MODIFIED		= 304,
-	HTTP_TEMPORARY_REDIRECT 	= 307,
-	HTTP_NOT_FOUND			= 404,
-	HTTP_REQUEST_ENTITY_TOO_LARGE   = 413,
-	HTTP_INTERNAL_SERVER_ERROR	= 500,
-	HTTP_NOT_IMPLEMENTED		= 501,
-	HTTP_BAD_GATEWAY		= 502
+    HTTP_CONTINUE                           = 100,
+    HTTP_SWITCHING_PROTOCOLS                = 101,
+    HTTP_OK                                 = 200,
+    HTTP_CREATED                            = 201,
+    HTTP_ACCEPTED                           = 202,
+    HTTP_NON_AUTHORATIVE_INFORMATION        = 203,
+    HTTP_NO_CONTENT                         = 204,
+    HTTP_RESET_CONTENT                      = 205,
+    HTTP_PARTIAL_CONTENT                    = 206,
+    HTTP_MULTIPLE_CHOICES                   = 300,
+    HTTP_MOVED_PERMANENTLY                  = 301,
+    HTTP_FOUND                              = 302,
+    HTTP_SEE_OTHER                          = 303,
+    HTTP_NOT_MODIFIED                       = 304,
+    HTTP_USE_PROXY                          = 305,
+    /* HTTP 306 is not used and reserved */
+    HTTP_TEMPORARY_REDIRECT                 = 307,
+    HTTP_BAD_REQUEST                        = 400,
+    HTTP_UNAUTHORIZED                       = 401,
+    HTTP_PAYMENT_REQUIRED                   = 402,
+    HTTP_FORBIDDEN                          = 403,
+    HTTP_NOT_FOUND                          = 404,
+    HTTP_BAD_METHOD                         = 405,
+    HTTP_NOT_ACCEPTABLE                     = 406,
+    HTTP_PROXY_AUTHENTICATION_REQUIRED      = 407,
+    HTTP_REQUEST_TIMEOUT                    = 408,
+    HTTP_CONFLICT                           = 409,
+    HTTP_GONE                               = 410,
+    HTTP_LENGTH_REQUIRED                    = 411,
+    HTTP_PRECONDITION_FAILED                = 412,
+    HTTP_REQUEST_ENTITY_TOO_LARGE           = 413,
+    HTTP_REQUEST_URI_TOO_LARGE              = 414,
+    HTTP_UNSUPPORTED_MEDIA_TYPE             = 415,
+    HTTP_REQUESTED_RANGE_NOT_SATISFIABLE    = 416,
+    HTTP_EXPECTATION_FAILED                 = 417,
+    HTTP_INTERNAL_SERVER_ERROR              = 500,
+    HTTP_NOT_IMPLEMENTED                    = 501,
+    HTTP_BAD_GATEWAY                        = 502,
+    HTTP_SERVICE_UNAVAILABLE                = 503,
+    HTTP_GATEWAY_TIMEOUT                    = 504,
+    HTTP_HTTP_VERSION_NOT_SUPPORTED         = 505
 };
 
 /*
@@ -147,6 +228,47 @@ void http_init(void);
 void http_shutdown(void);
 
 
+/***********************************************************************
+ * HTTP URL parsing.
+ */
+
+/*
+ * A structure describing a full URL with it's components.
+ */
+typedef struct {
+    Octstr *url;
+    Octstr *scheme;
+    Octstr *host;
+    unsigned long port;
+    Octstr *user;
+    Octstr *pass;
+    Octstr *path;
+    Octstr *query;
+    Octstr *fragment;
+} HTTPURLParse;
+
+/* 
+ * Create an URL parsing structure.
+ */
+HTTPURLParse *http_urlparse_create(void);
+
+/* 
+ * Destroy an URL parsing structure. 
+ */
+void http_urlparse_destroy(HTTPURLParse *p);
+
+/* 
+ * Parse the given URL and return a parsed struct containing all
+ * parsed components. If parsing failed, returns NULL.
+ */
+HTTPURLParse *parse_url(Octstr *url);
+
+/* 
+ * Dump the parsed struct to debug log level. 
+ */
+void parse_dump(HTTPURLParse *p);
+
+
 /***********************************************************************
  * HTTP proxy interface.
  */
@@ -160,8 +282,8 @@ void http_shutdown(void);
  * http_close_proxy closes the current proxy connection, after any
  * pending requests have been served.
  */
-void http_use_proxy(Octstr *hostname, int port, List *exceptions,
-    	    	    Octstr *username, Octstr *password);
+void http_use_proxy(Octstr *hostname, int port, int ssl, List *exceptions,
+    	    	    Octstr *username, Octstr *password, Octstr *exceptions_regex);
 void http_close_proxy(void);
 
 
@@ -169,6 +291,16 @@ void http_close_proxy(void);
  * HTTP client interface.
  */
 
+/*
+ * Define interface from which all http requestes will be served
+ */
+void http_set_interface(const Octstr *our_host);
+
+/**
+ * Define timeout in seconds for which HTTP clint will wait for
+ * response. Set -1 to disable timeouts.
+ */
+void http_set_client_timeout(long timeout);
 
 /*
  * Functions for doing a GET request. The difference is that _real follows
@@ -179,9 +311,9 @@ void http_close_proxy(void);
  *
  * XXX these are going away in the future
  */
-int http_get_real(Octstr *url, List *request_headers, Octstr **final_url,
-		  List **reply_headers, Octstr **reply_body);
-
+int http_get_real(int method, Octstr *url, List *request_headers, 
+                  Octstr **final_url, List **reply_headers, 
+                  Octstr **reply_body);
 
 /*
  * An identification for a caller of HTTP. This is used with
@@ -234,8 +366,8 @@ void http_caller_signal_shutdown(HTTPCaller *caller);
  * is no ssl authentication, unless you have set a global one with
  * use_global_certkey_file() from conn.c.
  */
-void http_start_request(HTTPCaller *caller, Octstr *url, List *headers,
-    	    	    	Octstr *body, int follow, void *id, 
+void http_start_request(HTTPCaller *caller, int method, Octstr *url, 
+                        List *headers, Octstr *body, int follow, void *id, 
     	    	    	Octstr *certkeyfile); 
 
 
@@ -244,9 +376,12 @@ void http_start_request(HTTPCaller *caller, Octstr *url, List *headers,
  * (the one passed to http_start request if non-NULL) or NULL if
  * http_caller_signal_shutdown has been called and there are no queued results.
  */
-void *http_receive_result(HTTPCaller *caller, int *status, Octstr **final_url,
-    	    	    	 List **headers, Octstr **body);
+void *http_receive_result_real(HTTPCaller *caller, int *status, Octstr **final_url,
+    	    	    	 List **headers, Octstr **body, int blocking);
 
+/* old compatibility mode, always blocking */
+#define http_receive_result(caller, status, final_url, headers, body) \
+    http_receive_result_real(caller, status, final_url, headers, body, 1)
 
 /***********************************************************************
  * HTTP server interface.
@@ -264,8 +399,16 @@ typedef struct HTTPClient HTTPClient;
  * Open an HTTP server at a given port. Return -1 for errors (invalid
  * port number, etc), 0 for OK. This will also start a background thread
  * to listen for connections to that port and read the requests from them.
+ * Second boolean variable indicates if the HTTP server should be started 
+ * for SSL-enabled connections.
  */
-int http_open_port(int port);
+int http_open_port(int port, int ssl);
+
+
+/*
+ * Same as above, but bind to a specific interface.
+ */
+int http_open_port_if(int port, int ssl, Octstr *interface);
 
 
 /*
@@ -352,7 +495,7 @@ void http_header_get(List *headers, long i, Octstr **name, Octstr **value);
 List *http_header_duplicate(List *headers);
 void http_header_pack(List *headers);
 void http_append_headers(List *to, List *from);
-Octstr *http_header_value(Octstr *header);
+Octstr *http_header_value(List *headers, Octstr *header);
 
 
 /*
@@ -428,7 +571,10 @@ void http_header_mark_transformation(List *headers, Octstr *new_body,
  * as a new Octet string, which the caller must free. Return NULL for
  * not found.
  */
-Octstr *http_header_find_first(List *headers, char *name);
+Octstr *http_header_find_first_real(List *headers, char *name, 
+                                    const char *file, long line, const char *func);
+#define http_header_find_first(headers, name) \
+    gw_claim_area(http_header_find_first_real((headers), (name), __FILE__, __LINE__, __func__))
 List *http_header_find_all(List *headers, char *name);
 
 
@@ -440,7 +586,9 @@ void http_header_get_content_type(List *headers, Octstr **type,
 
 
 /*
- * Do the headers indicate that MIME type `type' is accepted?
+ * Check if a specific mime-type can be handled by a client. This is
+ * indicated via 'Accept' headers. Returns 1 if the mime-type is acceptable,
+ * otherwise 0.
  */
 int http_type_accepted(List *headers, char *type);
 
@@ -450,9 +598,20 @@ int http_type_accepted(List *headers, char *type);
  */
 void http_header_dump(List *headers);
 
+/* 
+ * Ditto with cgi variables. Do not panic, when an empty are found from the 
+ * list.
+ */
+void http_cgivar_dump(List *cgiargs);
 
 /*
- * Check for acceptable charset
+ * As above function except that dump appended to Octstr.
+ */
+void http_cgivar_dump_into(List *cgiargs, Octstr *os);
+
+/*
+ * Check if the passed charset is in the 'Accept-Charset' header list
+ * alues of the client. Returns 1 if the charset is acceptable, otherwise 0.
  */
 int http_charset_accepted(List *headers, char *charset);
 
@@ -463,6 +622,32 @@ int http_charset_accepted(List *headers, char *charset);
 void http_add_basic_auth(List *headers, Octstr *username, Octstr *password);
 
 
+/* 
+ * Many HTTP field elements can take parameters in a standardized
+ * form: parameters appear after the main value, each is introduced
+ * by a semicolon (;), and consists of a key=value pair or just
+ * a key, where the key is a token and the value is either a token
+ * or a quoted-string.
+ * The main value itself is a series of tokens, separators, and
+ * quoted-strings.
+ *
+ * This function will take such a field element, and look for the 
+ * value of a specific key, which is then returned. If the key
+ * is not found within the header value NULL is returned.
+ * 
+ * BEWARE: value is *only* the header value, not the whole header with
+ * field name.
+ * 
+ * Example:
+ *    * assume to have "Content-Type: application/xml; charset=UTF-8" 
+ *    * within List *headers 
+ *   value = http_header_value(headers, octstr_imm("Content-Type"))
+ *   val = http_get_header_parameter(value, octstr_imm("charset"));
+ * will return "UTF-8" to lvalue.
+ */
+Octstr *http_get_header_parameter(Octstr *value, Octstr *parameter);
+
+
 /*
  * Return the general class of a status code.  For example, all
  * 2xx codes are HTTP_STATUS_SUCCESSFUL.  See the list at the top
@@ -470,4 +655,17 @@ void http_add_basic_auth(List *headers, Octstr *username, Octstr *password);
  */
 int http_status_class(int code);
 
+
+/*
+ * Return the HTTP_METHOD_xxx enum code for a Octstr containing 
+ * the HTTP method name.
+ */
+int http_name2method(Octstr *method);
+
+
+/*
+ * Return the char containing the HTTP method name.
+ */
+char *http_method2name(int method);
+
 #endif
