@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /*
  * octstr.h - Octet strings
  *
@@ -64,17 +120,21 @@ void octstr_shutdown(void);
  * Create an octet string from a NUL-terminated C string. Return pointer to
  * the new object.
  */
-Octstr *octstr_create_real(const char *cstr);
-#define octstr_create(cstr) gw_claim_area(octstr_create_real((cstr)))
-
+Octstr *octstr_create_real(const char *cstr, const char *file, long line,
+                           const char *func);
+#define octstr_create(cstr) \
+    (Octstr*)gw_claim_area(octstr_create_real((cstr), __FILE__, __LINE__, __func__))
 
 /*
  * Create an octet string from arbitrary binary data. The length of the
  * data is given, so it can contain NUL characters.
  */
-Octstr *octstr_create_from_data_real(const char *data, long len);
-#define octstr_create_from_data(data, len)\
-    gw_claim_area(octstr_create_from_data_real((data), (len)))
+Octstr *octstr_create_from_data_real(const char *data, long len, const char *file,
+                                     long line, const char *func);
+#define octstr_create_from_data(data, len) \
+    (Octstr*)gw_claim_area(octstr_create_from_data_real((data), (len), __FILE__, __LINE__, __func__))
+#define octstr_create_from_data_trace(data, len, file, line, func) \
+    (Octstr*)gw_claim_area(octstr_create_from_data_real(data, len, file, line, func))
 
 
 /*
@@ -96,7 +156,7 @@ void octstr_destroy(Octstr *ostr);
 
 /*
  * Destroy an octet string. Wrapper around octstr_destroy that is callable
- * via list_destroy.
+ * via gwlist_destroy.
  */
 void octstr_destroy_item(void *os);
 
@@ -104,7 +164,7 @@ void octstr_destroy_item(void *os);
 /*
  * Return the length of (number of octets in) an object string.
  */
-long octstr_len(Octstr *ostr);
+long octstr_len(const Octstr *ostr);
 
 
 /*
@@ -113,16 +173,19 @@ long octstr_len(Octstr *ostr);
  * octet string is created. If `from+len' is after the end of `ostr', 
  * `len' is reduced appropriately.
  */
-Octstr *octstr_copy_real(Octstr *ostr, long from, long len);
+Octstr *octstr_copy_real(const Octstr *ostr, long from, long len, const char *file,
+                         long line, const char *func);
 #define octstr_copy(ostr, from, len) \
-    gw_claim_area(octstr_copy_real((ostr), (from), (len)))
+    gw_claim_area(octstr_copy_real((ostr), (from), (len), __FILE__, __LINE__, __func__))
 
 
 /*
  * Copy all of an octet string.
  */
-Octstr *octstr_duplicate_real(Octstr *ostr);
-#define octstr_duplicate(ostr) gw_claim_area(octstr_duplicate_real((ostr)))
+Octstr *octstr_duplicate_real(const Octstr *ostr, const char *file, long line,
+                              const char *func);
+#define octstr_duplicate(ostr) \
+    gw_claim_area(octstr_duplicate_real((ostr), __FILE__, __LINE__, __func__))
 
 
 /*
@@ -137,7 +200,7 @@ Octstr *octstr_cat(Octstr *ostr1, Octstr *ostr2);
  * value has a range of 0..255 for valid positions, and -1 if `pos' is
  * after the end of the octet string.
  */
-int octstr_get_char(Octstr *ostr, long pos);
+int octstr_get_char(const Octstr *ostr, long pos);
 
 
 /*
@@ -165,7 +228,7 @@ void octstr_get_many_chars(char *buf, Octstr *ostr, long pos, long len);
  *
  * If the octet string is empty, an empty C string is returned, not NULL.
  */
-char *octstr_get_cstr_real(Octstr *ostr, const char *file, long line,
+char *octstr_get_cstr_real(const Octstr *ostr, const char *file, long line,
     	    	    	   const char *func);
 #define octstr_get_cstr(ostr) \
     (octstr_get_cstr_real(ostr, __FILE__, __LINE__, __func__))
@@ -220,6 +283,9 @@ void octstr_base64_to_binary(Octstr *ostr);
  */
 long octstr_parse_long(long *number, Octstr *ostr, long pos, int base);
 
+/* As above but parses and assigns double number. */
+long octstr_parse_double(double *number, Octstr *ostr, long pos);
+
 
 /* Run the 'filter' function over each character in the specified range.
  * Return 1 if the filter returned true for all characters, otherwise 0.
@@ -238,13 +304,20 @@ int octstr_check_range(Octstr *ostr, long pos, long len,
 void octstr_convert_range(Octstr *ostr, long pos, long len, 
     	    	    	  octstr_func_t map);
 
+/*
+ * Use the octstr_convert_range() with make_printable() to ensure
+ * every char in the octstr can be printed in the current locale. Each
+ * character that is NOT printable is converted to a '.' (dot).
+ */
+void octstr_convert_printable(Octstr *ostr);
+
 
 /*
  * Compare two octet strings, returning 0 if they are equal, negative if
  * `ostr1' is less than `ostr2' (when compared octet-value by octet-value),
  * and positive if greater.
  */
-int octstr_compare(Octstr *ostr1, Octstr *ostr2);
+int octstr_compare(const Octstr *ostr1, const Octstr *ostr2);
 
 
 /*
@@ -252,20 +325,32 @@ int octstr_compare(Octstr *ostr1, Octstr *ostr2);
  * Note that this probably doesn't work for Unicode, but should work
  * for such 8-bit character sets as are supported by libc.
  */
-int octstr_case_compare(Octstr *ostr1, Octstr *ostr2);
+int octstr_case_compare(const Octstr *ostr1, const Octstr *ostr2);
 
 
 /*
  * as above, but comparing is done only up to n bytes
  */
-int octstr_ncompare(Octstr *ostr1, Octstr *ostr2, long n);
+int octstr_ncompare(const Octstr *ostr1, const Octstr *ostr2, long n);
 
 
 /*
  * Same as octstr_compare, but compares the content of the octet string to 
  * a C string.
  */
-int octstr_str_compare(Octstr *ostr1, const char *str);
+int octstr_str_compare(const Octstr *ostr1, const char *str);
+
+
+/*
+ * Like octstr_str_compare, except compares bytes without case sensitifity.
+ */
+int octstr_str_case_compare(const Octstr *ostr1, const char *str);
+ 
+
+/*
+ * Same as octstr_str_compare, but comparing is done only up to n bytes.
+ */
+int octstr_str_ncompare(const Octstr *ostr, const char *str, long n);
 
 
 /*
@@ -278,7 +363,14 @@ int octstr_print(FILE *f, Octstr *ostr);
  * Search the character from octet string starting from position pos. Returns 
  * the position (index) of the char in string, -1 if not found.
  */
-int octstr_search_char(Octstr *ostr, int ch, long pos);
+int octstr_search_char(const Octstr *ostr, int ch, long pos);
+
+
+/*
+ * Search several character from octet string starting from position pos. Returns 
+ * the position (index) of the first char found in string, -1 if none was found.
+ */
+int octstr_search_chars(const Octstr *ostr, const Octstr *chars, long pos);
 
 
 /*
@@ -286,14 +378,18 @@ int octstr_search_char(Octstr *ostr, int ch, long pos);
  * Return the start position (index) of 'needle' in 'haystack'.
  * Return -1 if not found.
  */
-int octstr_search(Octstr *haystack, Octstr *needle, long pos);
+int octstr_search(const Octstr *haystack, const Octstr *needle, long pos);
 
 
 /*
  * Like octstr_search, but ignores 8-bit byte case.
  */
-int octstr_case_search(Octstr *haystack, Octstr *needle, long pos);
+int octstr_case_search(const Octstr *haystack, const Octstr *needle, long pos);
 
+/*
+ * Like octstr_case_search, but searchs only first n octets.
+ */
+int octstr_case_nsearch(const Octstr *haystack, const Octstr *needle, long pos, long n);
 
 /*
  * Write contents of octet string to a file, in human readable form. 
@@ -328,7 +424,7 @@ int octstr_append_from_socket(Octstr *ostr, int socket);
  * Insert one octet string into another. `pos' gives the position
  * in `ostr1' where `ostr2' should be inserted.
  */
-void octstr_insert(Octstr *ostr1, Octstr *ostr2, long pos);
+void octstr_insert(Octstr *ostr1, const Octstr *ostr2, long pos);
 
 
 /*
@@ -340,6 +436,11 @@ void octstr_insert(Octstr *ostr1, Octstr *ostr2, long pos);
  */
 void octstr_insert_data(Octstr *ostr, long pos, const char *data, long len);
 
+/*
+ * Similar as previous, expect that now a single character is inserted.
+ */
+void octstr_insert_char(Octstr *ostr, long pos, const char c);
+
 
 /*
  * Append characters from C array at the tail of an octet string.
@@ -350,7 +451,7 @@ void octstr_append_data(Octstr *ostr, const char *data, long len);
 /*
  * Append a second octstr to the first.
  */
-void octstr_append(Octstr *ostr1, Octstr *ostr2);
+void octstr_append(Octstr *ostr1, const Octstr *ostr2);
 
 
 /*
@@ -377,6 +478,10 @@ void octstr_truncate(Octstr *ostr, int new_len);
  */
 void octstr_strip_blanks(Octstr *ostr);
 
+/*
+ * Strip CR and LF from start and end of a octet string.
+ */
+void octstr_strip_crlfs(Octstr *ostr);
 
 /*
  * Strip non-alphanums from start and end of a octet string.
@@ -403,30 +508,50 @@ void octstr_delete(Octstr *ostr1, long pos, long len);
 Octstr *octstr_read_file(const char *filename);
 
 
+/*
+ * Read the contents of a file descriptor pipe to an octet string. 
+ * Return pointer to octet string.
+ */
+Octstr *octstr_read_pipe(FILE *f);
+
+
 /*
  * Split an octet string into words at whitespace, and return a list
  * containing the new octet strings.
  */
-List *octstr_split_words(Octstr *ostr);
+List *octstr_split_words(const Octstr *ostr);
 
 
 /*
  * Split an octet string into substrings at every occurence of `sep'.
  * Return List with the substrings.
  */
-List *octstr_split(Octstr *os, Octstr *sep);
+List *octstr_split(const Octstr *os, const Octstr *sep);
 
 
 /*
- * Compare two octet strings in a manner suitable for list_search.
+ * Compare two octet strings in a manner suitable for gwlist_search.
  */
 int octstr_item_match(void *item, void *pattern);
 
 
 /*
- * Print debugging information about octet string.
+ * Same as above, except compares bytes without case sensitivity
  */
-void octstr_dump(Octstr *ostr, int level);
+int octstr_item_case_match(void *item, void *pattern);
+
+
+/*
+ * Print debugging information about octet string. This is abstracted to the
+ * various log levels we have: GW_DEBUG, GW_INFO, GW_WARNING, GW_ERROR
+ * 
+ * If a third parameter in the argument list is given, we will dump the
+ * octstr in that log level instead of the default GW_DEBUG level.
+ */
+void octstr_dump_real(const Octstr *ostr, int level, ...);
+#define octstr_dump(ostr, level, ...) \
+    octstr_dump_real(ostr, level, GW_DEBUG, ##__VA_ARGS__)
+
 
 /*
  * Write the contents of an octet string to the debug log.
@@ -435,17 +560,20 @@ void octstr_dump(Octstr *ostr, int level);
  */
 void octstr_dump_short(Octstr *ostr, int level, const char *name);
 
+
 /*
  * decode url-encoded octstr in-place.
  * Return 0 if all went fine, or -1 if there was some garbage
  */
 int octstr_url_decode(Octstr *ostr);
 
+
 /*
  * URL encode the argument string in place.
  */
 void octstr_url_encode(Octstr *ostr);
 
+
 /*
  * Treat the octstr as an unsigned array of bits, most significant bit
  * first, and return the indicated bit range as an integer.  numbits
@@ -454,6 +582,7 @@ void octstr_url_encode(Octstr *ostr);
  */
 long octstr_get_bits(Octstr *ostr, long bitpos, int numbits);
 
+
 /*
  * Treat the octstr as an unsigned array of bits, most significant bit
  * first, and set the indicated bit range to the given value.  numbits
@@ -533,20 +662,23 @@ void octstr_append_decimal(Octstr *ostr, long value);
  *				inside the string is included in the
  *				output
  *
- *                      E       Octstr *
- *                              output as character string, except that
- *                              contents are URL-encoded when need to. Note
- *                              that trunctae is done afterwards and can
- *                              cut escape '%EE' in half
+ *			E	Octstr *
+ *				output as character string, except that
+ *				contents are URL-encoded when need to. Note
+ *				that trunctae is done afterwards and can
+ *				cut escape '%EE' in half
+ *
+ *			H	Octstr *
+ *				output as character string, except that
+ *				contents are HEX-encoded in uppercase
  */
 Octstr *octstr_format(const char *fmt, ...);
 
-
 /*
  * Like octstr_format, but takes the argument list as a va_list.
  */
-Octstr *octstr_format_valist(const char *fmt, va_list args);
-
+Octstr *octstr_format_valist_real(const char *fmt, va_list args);
+#define octstr_format_valist(fmt, args) gw_claim_area(octstr_format_valist_real(fmt, args))
 
 /*
  * Like octstr_format, but appends output to an existing octet
@@ -561,5 +693,53 @@ void octstr_format_append(Octstr *os, const char *fmt, ...);
  */
 unsigned long octstr_hash_key(Octstr *ostr);
 
+/*
+ * return an Octstr encoded in charset named tocode created from the data
+ * in the Octstr orig that is encoded in the charset fromcode.
+ */
+int octstr_recode(Octstr *tocode, Octstr *fromcode, Octstr *orig);
+
+/*
+ * Strip all occurence of char ch from start of Octstr
+ */
+void octstr_strip_char(Octstr *text, char ch);
+
+/*
+ * Check if ostr is numeric
+ */
+int octstr_isnum(Octstr *ostr1);
+
+/*
+ * Replace all occurences of needle with repl within haystack
+ */
+void octstr_replace(Octstr *haystack, Octstr *needle, Octstr *repl);
+
+/*
+ * Symbolize hex string '78797a' becomes '%78%79%7a'
+ */
+int octstr_symbolize(Octstr *ostr);
+
+/*
+ * Remove all occurrences of 'needle' within 'haystack'.
+ */
+void octstr_delete_matching(Octstr *haystack, Octstr *needle);
+
+/*
+ * Return 1, if octstr 'os' contains only hex chars, 0 otherwise.
+ */
+int octstr_is_all_hex(Octstr *os);                                                    
+                                                    
+/*
+ * make data HTML safe by converting appropriate characters to HTML entities.
+ * conversion is done in place
+ */
+void octstr_convert_to_html_entities(Octstr* input);
+
+/*
+ * convert HTML safe data back to binary data by replacing HTML entities with their
+ * respective character values.
+ * conversion is done in place
+ */
+void octstr_convert_from_html_entities(Octstr* input);
 
 #endif
