@@ -1,3 +1,59 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 /* test_cimd2.c - fake cimd2 smsc
  *
  * This program pretends to be an CIMD 2 SMS center, accessible via IP.
@@ -439,18 +495,18 @@ static void handle_submit(Octstr *packet, Octstr *out, int sequence) {
 	long tariff_class = eat_int_parm(packet, 64, 2);
 	long service_desc = eat_int_parm(packet, 65, 1);
 	long priority = eat_int_parm(packet, 67, 1);
-	List *other_dests = list_create();
+	List *other_dests = gwlist_create();
 	Octstr *tmp;
 
 	while ((tmp = eat_string_parm(packet, 21, 20)))
-		list_append(other_dests, tmp);
+		gwlist_append(other_dests, tmp);
 
 	if (logging == LOG_packets) {
 		int i;
 		printf("RCV: Submit to %s", octstr_get_cstr(dest_addr));
-		for (i = 0; i < list_len(other_dests); i++) {
+		for (i = 0; i < gwlist_len(other_dests); i++) {
 			printf(", %s",
-				octstr_get_cstr(list_get(other_dests, i)));
+				octstr_get_cstr(gwlist_get(other_dests, i)));
 		}
 		printf("\n");
 
@@ -504,7 +560,7 @@ static void handle_submit(Octstr *packet, Octstr *out, int sequence) {
 
 	if (!dest_addr) {
 		send_error(out, 53, sequence, "300", "no destination");
-	} else if (list_len(other_dests) > 0) {
+	} else if (gwlist_len(other_dests) > 0) {
 		send_error(out, 53, sequence, "301", "too many destinations");
 	/* TODO: Report many other possible errors here */
 	} else {
@@ -526,7 +582,7 @@ static void handle_submit(Octstr *packet, Octstr *out, int sequence) {
 	octstr_destroy(textb);
 	octstr_destroy(valid_abs);
 	octstr_destroy(delivery_abs);
-	list_destroy(other_dests, octstr_destroy_item);
+	gwlist_destroy(other_dests, octstr_destroy_item);
 }
 
 static void handle_enquire(Octstr *packet, Octstr *out, int sequence) {
@@ -964,7 +1020,7 @@ static int wait_for_client(int port) {
 	int clientfd;
 	Octstr *addr;
 
-	listenfd = make_server_socket(port);
+	listenfd = make_server_socket(port, NULL);
 	if (listenfd < 0) {
 		fprintf(stderr, "%s: failed to open socket at port %d\n",
 			progname, port);
