@@ -1,7 +1,63 @@
+/* ==================================================================== 
+ * The Kannel Software License, Version 1.0 
+ * 
+ * Copyright (c) 2001-2009 Kannel Group  
+ * Copyright (c) 1998-2001 WapIT Ltd.   
+ * All rights reserved. 
+ * 
+ * Redistribution and use in source and binary forms, with or without 
+ * modification, are permitted provided that the following conditions 
+ * are met: 
+ * 
+ * 1. Redistributions of source code must retain the above copyright 
+ *    notice, this list of conditions and the following disclaimer. 
+ * 
+ * 2. Redistributions in binary form must reproduce the above copyright 
+ *    notice, this list of conditions and the following disclaimer in 
+ *    the documentation and/or other materials provided with the 
+ *    distribution. 
+ * 
+ * 3. The end-user documentation included with the redistribution, 
+ *    if any, must include the following acknowledgment: 
+ *       "This product includes software developed by the 
+ *        Kannel Group (http://www.kannel.org/)." 
+ *    Alternately, this acknowledgment may appear in the software itself, 
+ *    if and wherever such third-party acknowledgments normally appear. 
+ * 
+ * 4. The names "Kannel" and "Kannel Group" must not be used to 
+ *    endorse or promote products derived from this software without 
+ *    prior written permission. For written permission, please  
+ *    contact org@kannel.org. 
+ * 
+ * 5. Products derived from this software may not be called "Kannel", 
+ *    nor may "Kannel" appear in their name, without prior written 
+ *    permission of the Kannel Group. 
+ * 
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED 
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES 
+ * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE 
+ * DISCLAIMED.  IN NO EVENT SHALL THE KANNEL GROUP OR ITS CONTRIBUTORS 
+ * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,  
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT  
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR  
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,  
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE  
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. 
+ * ==================================================================== 
+ * 
+ * This software consists of voluntary contributions made by many 
+ * individuals on behalf of the Kannel Group.  For more information on  
+ * the Kannel Group, please see <http://www.kannel.org/>. 
+ * 
+ * Portions of this software are based upon software originally written at  
+ * WapIT Ltd., Helsinki, Finland for the Kannel project.  
+ */ 
+
 #include <unistd.h>
 #include "gwlib/gwlib.h"
-#include "gw/smpp_pdu.h"
-
+#include "gw/smsc/smpp_pdu.h"
+#include <string.h>
 
 /***********************************************************************
  * Configurable stuff.
@@ -138,37 +194,37 @@ static void eq_destroy_event(Event *e)
 
 static EventQueue *eq_create(void)
 {
-    return list_create();
+    return gwlist_create();
 }
 
 
 static void eq_add_producer(EventQueue *eq)
 {
-    list_add_producer(eq);
+    gwlist_add_producer(eq);
 }
 
 
 static void eq_remove_producer(EventQueue *eq)
 {
-    list_remove_producer(eq);
+    gwlist_remove_producer(eq);
 }
 
 
 static void eq_destroy(EventQueue *eq)
 {
-    list_destroy(eq, NULL);
+    gwlist_destroy(eq, NULL);
 }
 
 
 static void eq_append(EventQueue *eq, Event *e)
 {
-    list_produce(eq, e);
+    gwlist_produce(eq, e);
 }
 
 
 static Event *eq_extract(EventQueue *eq)
 {
-    return list_consume(eq);
+    return gwlist_consume(eq);
 }
 
 
@@ -342,7 +398,7 @@ static void smpp_emu_reader(void *arg)
 		    error(0, "Client sent garbage, closing connection.");
 		    goto error;
 		} else if (len == 0) {
-		    if (conn_eof(p->conn) || conn_read_error(p->conn))
+		    if (conn_eof(p->conn) || conn_error(p->conn))
 		    	goto error;
 		    break;
 		}
@@ -361,7 +417,7 @@ static void smpp_emu_reader(void *arg)
 		    smpp_pdu_destroy(pdu);
 		}
 		octstr_destroy(os);
-	    } else if (conn_eof(p->conn) || conn_read_error(p->conn))
+	    } else if (conn_eof(p->conn) || conn_error(p->conn))
 	    	goto error;
 	    else
 		break;
@@ -393,7 +449,7 @@ static void smpp_emu(void *arg)
     /*
      * Wait for SMPP clients.
      */
-    fd = make_server_socket(smpp_port);
+    fd = make_server_socket(smpp_port, NULL);
     if (fd == -1)
     	panic(0, "Couldn't create SMPP listen port.");
     
@@ -408,7 +464,7 @@ static void smpp_emu(void *arg)
 	    (void) close(new_fd);
 	} else {
 	    thread[num_threads] = gw_malloc(sizeof(*thread[0]));
-    	    thread[num_threads]->conn = conn_wrap_fd(new_fd);
+    	    thread[num_threads]->conn = conn_wrap_fd(new_fd, 0);
 	    thread[num_threads]->eq = eq;
 	    thread[num_threads]->quit = 0;
 	    thread[num_threads]->writer_id = -1;
@@ -574,8 +630,9 @@ static long httpd_emu_tid = -1;
 static void httpd_emu_create(EventQueue *eq)
 {
     struct httpd_emu_arg *arg;
+    int ssl = 0;   /* indicate if SSL-enabled server should be used */
 
-    if (http_open_port(http_port) == -1)
+    if (http_open_port(http_port, ssl) == -1)
     	panic(0, "Can't open HTTP server emulator port %ld.", http_port);
 
     gw_assert(httpd_emu_tid == -1);
@@ -644,8 +701,8 @@ static void kill_kannel(void)
     	    	    	admin_port, admin_password);
     req_headers = http_create_empty_headers();
     http_header_add(req_headers, "Content-Type", "text/plain");
-    ret = http_get_real(url, req_headers, &final_url, &reply_headers, 
-    	    	    	&reply_body);
+    ret = http_get_real(HTTP_METHOD_GET, url, req_headers, &final_url, 
+                        &reply_headers, &reply_body);
     if (ret != -1) {
     	octstr_destroy(final_url);
 	http_destroy_headers(reply_headers);
@@ -873,7 +930,7 @@ int main(int argc, char **argv)
 	}
     }
 
-    for (i = 0; i < sizeof(tab) / sizeof(tab[0]); ++i) {
+    for (i = 0; (size_t) i < sizeof(tab) / sizeof(tab[0]); ++i) {
 	if (strcmp(main_name, tab[i].name) == 0) {
 	    tab[i].func();
 	    break;
